<?php
if(str_replace( '\\', '/',$_SERVER['DOCUMENT_ROOT']) == PDBF_PATH) { define('PDBF_ABS_PATH', '');}
else { define('PDBF_ABS_PATH', PDBF_PATH); }

if(!session_id()){ session_start();}
class C_DatabaseForm extends C_DataGrid {
    private $form_property = array();
    private $form_width = '640px';
    private $form_height = '1000px';
    private $border = '1px solid #aaa';
    private $pk = null;


    /**
     * Advanced method to set form properties directly through an array.
     * Reference: http://www.trirand.com/jqgridwiki/doku.php?id=wiki:form_editing
     *
     * @param array $prop - Form properties
     * @return $this
     */
    public function set_form_property($prop=array()){
        $this->form_property = $prop;
        return $this;
    }

    /**
     * Set form element label
     *
     * @param $col - Database table column name
     * @param $label - Label
     * @return $this
     */
    public function set_label($col, $label){
        $this->set_col_title($col, $label);

        return $this;
    }

    /**
     * Set control dimension width and height. Note that this is not the same as set_form_dimension().
     * It is used with text type only. For column type text, the width is translated to "size", its height is ignored.
     * for text area, the width is translated to "cols" and height to "rows"
     *
     * @param $col - Database table column name
     * @param int $cols - If the column is a text box, it’s the size that represents number of characters.
     * If the column is a textarea, it’s the value for cols attribute in textarea.
     * @param int $rows - Optional. Number of rows in a textarea (only)
     * @return $this
     */
    public function set_dimension($col, $cols=30, $rows=6){
        $this->set_col_edit_dimension($col, $cols, $rows);

        return $this;
    }

    /**
     * Set form width and height. Overwrites parent definition for template.
     *
     * @param $f_width - Form width
     * @param string $f_height - Form height
     * @return $this|void
     */
    public function set_form_dimension($f_width, $f_height='100%'){
        parent::set_form_dimension($f_width,$f_height);

        $this->form_width = $f_width;
        $this->form_height = $f_height;
    }

    /**
     * Set control default value on the form
     *
     * @param $col - Database table column name
     * @param string $default - Default value
     * @return $this
     */
    public function set_default($col, $default=""){
        $this->set_col_default($col, $default);

        return $this;
    }

    /**
     * Set one more more columns as read only field
     *
     * @param array $cols - Column name(s)
     * @return $this
     */
    public function set_readonly($cols=array()){
        $cols_delimited = implode(',', $cols);   // set_col_readonly take a string
        $this->set_col_readonly($cols_delimited);

        return $this;
    }

    /**
     * @param array $cols - Columns name(s)
     * @return $this
     */
    public function set_required($cols=array()){
        $cols_delimited = implode(',', $cols);   // set_col_readonly take a string
        $this->set_col_required($cols_delimited);

        return $this;
    }

    /**
     * Hide a field on the form from displaying with CSS display:none. Note that the field still exists on the form but not visible.
     * Do not use this field to hide sensitive information such as password.
     *
     * @param $col - Database table column name
     * @return $this
     */
    public function set_hide($col){
        $this->set_col_hidden($col, false);

        return $this;
    }

    //
    /**
     * Placeholder text for HTML form element.
     *
     * @param $col - Database table column name
     * @param $ph_text - Placeholder text
     * @return $this
     */
    public function set_placeholder($col, $ph_text){
        $this->set_col_property($col, array("editoptions"=>array('placeholder'=>$ph_text)));

        return $this;
    }

    /*
    ref: http://www.w3.org/TR/html401/interact/forms.html
    ref: http://www.wufoo.com/features/
    ref: http://www.apphp.com/php-dataform/index.php?page=examples
    ref: http://www.trirand.net/demo/php/jqform/
    ref: http://www.trirand.com/jqgridwiki/doku.php?id=wiki:form_editing
    http://www.trirand.com/blog/phpjqgrid/examples/selection/selectedrow_client/default.php
    http://stackoverflow.com/questions/10320523/how-to-add-custom-buttons-to-jqgrid-add-edit-forms

    -- Requirements:
    New data entry
        #Form redirect after submit
        #Form stays after submit (default)
    #Edit existing data in the form
    #captcha (TODO - fix js path),
    #section break via title & hr or fieldset and legend,
    #tooltip (thru jquery tipsy) - HTML supported
    #data validation,
    #hidden control (using set_hidden),
    #autocomplete,
    #submit redirect
    #advanced property via array,
    #read only control,
    #disabled control (editable = false),
    #wysiwygj,
    #theme support!
    #default value,
    #checkbox list
    #radio list
    #placeholder,
    #theme support!
    local array support
    template (may need to push to version 2)
    popular form examples: login form, membership form, credit card form


    --- VERSION 2 ---
    array datasource support to submit to
        Google Spreadsheet
        or event a special URL to collect info similar to Wufoo. No DB is required. Great for customers.
        so form can also be embeded this way
    multiple input custom element (http://stackoverflow.com/questions/3054811/add-multiple-input-elements-in-a-custom-edit-type-field)
    ### file upload ####, VERSION 2!
    link to file,
    button image,
    rating control (wufoo),
    nested dropdown,
    remember me,
    logic & branching (wufoo),
    image display,
    image button,
    js event handling(e.g init focus, jqGridAddEditBeforeShowForm)
    submit confirmation (done through above),


    */

    // set mode DATA or FORM
    //public function set_mode($m = 'DATA'){
    //}

    //
    /**
     * Set form border using CSS stylesheet
     *
     * @param $border_style - border style in CSS
     * @return $this
     */
    public function set_border($border_style){
        $this->border = $border_style;

        return $this;
    }

    /**
     *
     *
     * @param $col - Database table column name
     * @param $ctrltype - HTML control type
     * @return $this
     */
    /**
     * Set HTML control type of a specific field. Select and Textare control types are automatically determined
     * by the database field char type based on its length. But you can set it otherwise.
     *
     * @param $col - Database table column name
     * @param $ctrltype - HTML control type. Supported types:
     *          text
                textarea
                select
                checkbox
                passwords
                button
                autocomplete
     *          wysiwyg
     * @param null $keyvalue_pair - Key value pair or SQL SELECT(select control type only)
     * @param bool $multiple - Multiple select (select control type only)
     * @return $this
     */
    public function set_ctrltype($col, $ctrltype, $keyvalue_pair=null, $multiple=false){
        if($ctrltype == 'wysiwyg'){
            $this->set_col_wysiwyg($col);
        }elseif($ctrltype == 'radio'){ // no predefined or custom ctrl type, need to implement our own
            $this->set_col_edittype($col, 'radio', $keyvalue_pair);
        }elseif($ctrltype == 'checkboxlist'){ // no predefined or custom ctrl type, need to implement our own
            $this->set_col_edittype($col, 'checkboxlist', $keyvalue_pair);
        }elseif($ctrltype == 'email'){
            $this -> set_editrule($col, "pdbf.validateEmail");
        }else{
            $this->set_col_edittype($col, $ctrltype, $keyvalue_pair, $multiple, null);
        }

        /* ADOdb manual indicates MySql max_length property doesn't return reliable data, I approved it true. Don't use it.
        $this_db            = $this->db;
        $results            = $this_db->select_limit($this->get_sql(),1, 1);
        $len = $this_db->field_len($results, $this_db->field_index($results, 'LastName'));
        echo 'len: '. $len;
        */

        return $this;
    }


    /***
     * Set max input length for a field
     *
     * @param $col - column name
     * @param $maxlength - integer
     * @return $this
     */
    public function set_maxlength($col, $maxlength){
       $this->set_col_property($col, array("editoptions"=>array("maxlength"=>$maxlength)));

       return $this;
    }


    /**
     * @param $col - Column name
     * @param $rule_func - Data rule validation JavaScript
     * @return $this
     */
    public function set_editrule($col, $rule_func){
        $this->set_col_customrule($col, $rule_func);

        return $this;
    }

    /**
     * Only include the header scripts once.
     */
    public function display_script_includeonce(){
        parent::display_script_includeonce();

        echo '<link rel="stylesheet" type="text/css" media="screen" href="'. PDBF_ABS_PATH .'/css/pdbf.css" />' ."\n";
        echo '<script src="'. PDBF_ABS_PATH .'/js/pdbf_edit.js" type="text/javascript"></script>' ."\n";

        echo '<link rel="stylesheet" type="text/css" href="'. PDBF_ABS_PATH .'/js/realperson/jquery.realperson.css">'. "\n";
        echo '<script type="text/javascript" src="'. PDBF_ABS_PATH .'/js/realperson/jquery.realperson.min.js"></script>' ."\n";

        echo '<script src="'. PDBF_ABS_PATH .'/js/mask/jquery.mask.min.js" type="text/javascript"></script>' ."\n";
        echo '<script src="'. PDBF_ABS_PATH .'/js/notify.min.js" type="text/javascript"></script>' ."\n";
    }

    /**
     * Display our database form
     *
     * @param bool $render_content - Whether to display the form or hold it in PHP buffer to be displayed later.
     */
    public function display($render_content=true){
        $action = '';

        parent::enable_edit("FORM", "CRUD");

        $this->before_script_end .= '$("#'. $this->get_sql_table() .'").closest(".ui-jqgrid").hide();'."\n";


        $hasRecord = false;
        
        if( !is_null($this->pk) ){

            $sql_key = $this->get_sql_key();
            
            $qstr = $this->db->quote_field($this->get_sql(), $sql_key[0], $this->pk);

            $result = $this->db->db_query($this->get_sql(). ' WHERE '. $qstr);
            
            $hasRecord = ($this->db->num_rows($result) != 0);

        }

        if(is_null($this->pk) || !$hasRecord){

            $action = '$("#add_'. $this->get_sql_table() .'").click();'; // show add form
        
        }else{
        
            $action  = '$("#'. $this->get_sql_table() .'").jqGrid("setSelection","'. $this->pk .'");'; // select an existing 
            $action .= '$("#edit_'. $this->get_sql_table() .'").click();'; // click the edit icon

        }
        
        // load initial form
        $this->before_script_end .= 'setTimeout(function(){'. $action .'
            $("div#edithd'. $this->get_sql_table() .'").css("display","none");
            $("div#editmod'. $this->get_sql_table() .'").css("border","'. $this->border .'");
            $("div#editmod'. $this->get_sql_table() .'").css("width","'. $this->form_width .'");
            $("div#editmod'. $this->get_sql_table() .'").addClass("paper_effect");

            $("form#FrmGrid_'. $this->get_sql_table() .'").css({"width":""});
            $("table#TblGrid_'. $this->get_sql_table() .'").css({"width":"'. $this->form_width .'","padding-right":"10px"});

            /*
            var $inputs = jQuery("#FrmGrid_'. $this->get_sql_table() .' :input");
                var values = {};
                $inputs.each(function() {
                    //console.log(values[this.name] = $(this).val());
                    if ($(this).attr("type") == "text"){
                        $(this).attr("value", $(this).val());
                    }else if (typeof $(this).attr("type") === "undefined"){
                        // console.log($(this).get(0).tagName);
                        if($(this).get(0).tagName.toLowerCase() === "select"){
                            sel = "#" + this.name + " option";
                            // console.log(sel + " : " + $(this).val());
                            $(sel).prop("selected", false).filter("[value=" + $(this).val() + "]").prop("selected", true); // TODO not working, but works when mannual execute in Console.
                        }
                    }

            });

            $("table#TblGrid_'. $this->get_sql_table() .'").replaceWith($("table#TblGrid_'. $this->get_sql_table() .'").html().
                replace(/<tbody/gi,"<ul").
                replace(/<tr/gi,"<li").
                replace(/<\/tr>/gi,"</li>").
                replace(/<td/gi,"<p").
                replace(/<\/td>/gi,"</p>").
                replace(/<th/gi,"<p").
                replace(/<\/th>/gi,"</p>").
                replace(/<\/tbody/gi,"</ul").
                replace(/&nbsp;/gi,""));
            $("#tr_LastName").addClass("leftHalf");$("#LastName").addClass("field text medium");
            $("#tr_FirstName").addClass("rightHalf");$("#FirstName").addClass("field text medium");
            $("#tr_Address").addClass("leftFourth");$("#Address").addClass("field text medium");
            $("#tr_City").addClass("middleFourth");$("#City").addClass("field text medium");
            $("#tr_Region").addClass("middleFourth");$("#Region").addClass("field text medium");
            $("#tr_PostalCode").addClass("rightFourth");$("#PostalCode").addClass("field text medium");
            $("#tr_ReportsTo").addClass("leftHalf");$("#ReportsTo").addClass("field select medium");
            */
        }, 200);'."\n";

/*
         $this->before_script_end .= '$.extend($.jgrid.edit, {
                bSubmit: "Save and Close",
                bCancel: "Cancel Me",
                closeAfterEdit: false,
                jqModal:true,
                viewPagerButtons:false,
                //recreateForm:true,
                closeOnEscape:false
            });';

        // event will not work because it will be overwritten by other beforeShowForm handler
        $this->before_script_end .= '$.extend($.jgrid.edit, {
                recreateForm: true,
                beforeShowForm: function(form) {
                    $(\'<tr class="FormData"><td class="CaptionTD ui-widget-content" colspan="2">\' +
                      \'<hr/><div style="padding:3px" class="ui-widget-header ui-corner-all">\' +
                      \'<b>Invice information (all about money):</b></div></td></tr>\')
                    .insertBefore($("#BirthDate", form));
                }
            });';

*/
        parent::display($render_content);
    }


    /**
     * Load an existing entry from DB.
     *
     * @param int $k - A database table primary key value. Zero to load the very entry.
     * @return $this
     */
    public function load_form($k=0){
        $this->pk = $k;

        if($k != 0) {
            $sql_key = $this->get_sql_key();
            $qstr = $this->db->quote_field($this->get_sql(), $sql_key[0], $k);
            $this->set_query_filter($qstr);
        }

        return $this;
    }

    /**
     * Redirect form after submit. It does not check whether the submit is success.
     *
     * @param $url - Redirect URL
     * @return $this
     */
    public function redirect_after_submit($url){
        $this -> add_event("jqGridAddEditAfterSubmit", "function (event, status, postData){window.location='". $url ."';}");

        return $this;
    }

    /**
     * Add caption to a subgroup
     *
     * @param $before_col - column name that caption goes BEFORE
     * @param $header - caption
     * @return $this
     */
    public function add_group_header($before_col, $header){
        $this->add_form_group_header($before_col, $header);

        return $this;
    }


    /**
     * Add CAPTCHA (“Completely Automated Public Turing test to tell Computers and Humans Apart”),
     * a type of challenge-response test used in computing to determine whether or not the user is human.
     *
     * phpDatabaseForm uses realperson jQuery plugin available under MIT license.
     *
     * @return $this
     */
    public function add_captcha(){
        $this -> add_column('Captcha', array('name'=>'_realperson', 'index'=>'_realperson', 'editable'=>true,
            'editrules'=>array('required'=>true,
                'custom'=>true,'custom_func'=>'###pdbf.validateCaptcha###')), 'Are you human?');

        return $this;

    }

    /**
     * Provides a general format to text field and enforce it's structure as the user types in values.
     * It gives the user an idea about how they should enter the data.
     *
     * phpDatabaseForm uses jQuery Mask Input created by Igor Escobar http://igorescobar.github.io/jQuery-Mask-Plugin/
     * available under MIT license.
     *
     * @param $col - Column name
     * @param $mask - Mask pattern
     * @param string $extra_prop - Optional. Addition properties e.g. "{placeholder: '(___)__-____'}". Note must single quote
     */
    public function set_mask($col, $mask, $extra_prop=''){
        $xprop = ($extra_prop=='')?'':','.$extra_prop; // add leading comma if not empty
        $this -> set_col_property($col, array("editoptions"=>array("dataInit"=>"###function (elem) { $(elem).mask('". $mask ."'". $xprop .");}###")));
    }

}